@@ -337,9 +337,6 @@ MicrobEalXshmRenderer::DrawRectToXshm(nsIDOMWindow *aWindow,
 
     ULOG("[%i,%i,%i,%i]:, nr[%i,%i,%i,%i]\n", (int)l, (int)t, (int)width, (int)height, nr.x, nr.y, nr.width, nr.height);
 
-    // Detach from shared memory data
-    engine->shmid = aData.id;
-
     PaintNotifyMessage msg =
         { (guint32) engine, aReqestId, (int)roundf(zooml), (int)roundf(zoomt),
           (int)roundf(zoomWidth), (int)roundf(zoomHeight), scrollMaxX+innerWidth,
@@ -379,6 +376,19 @@ MicrobEalXshmRenderer::byte_per_pixel(int depth)
   return byte_per_pix;
 }
 
+nsresult
+MicrobEalXshmRenderer::UpdateUIShmInfo(void)
+{
+  // Get Mozilla Web
+  GMozillaWeb* web = G_MOZILLA_WEB(mGWeb);
+  // Check Mozilla Web
+  NS_ENSURE_TRUE(web, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(web->web_bus, NS_ERROR_FAILURE);
+  ShmInfoMessage msg = { 0, mShmData.id, mShmData.width, mShmData.height, mShmData.depth };
+  g_web_bus_send_message(web->web_bus, G_MOZILLA_UI, SHM_INFO_MSG, &msg, sizeof(msg));
+  return NS_OK;
+}
+
 nsresult
 MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, GMozillaEngine *aEngine)
 {
@@ -390,6 +400,7 @@ MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, G
     mShmData.id = -1;
     mShmData.blocked = 0;
     mShmData.data = 0;
+    UpdateUIShmInfo();
     return NS_OK;
   }
   if (width == 0 && height == 0 && mShmData.id == -1) {
@@ -423,11 +434,7 @@ MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, G
 
   if (mShmData.id != -1 && width <= mShmData.width && height <= mShmData.height && depth == mShmData.depth) {
     ULOG("SHM already created %i", mShmData.id);
-    if (aEngine && mGWeb) {
-      aEngine->shmid = mShmData.id;
-      ShmInfoMessage msg = { (guint32) aEngine, mShmData.id, mShmData.width, mShmData.height, mShmData.depth };
-      g_web_bus_send_message(G_MOZILLA_WEB(mGWeb)->web_bus, G_MOZILLA_UI, SHM_INFO_MSG, &msg, sizeof(msg));
-    }
+    UpdateUIShmInfo();
     return NS_OK;
   }
 
@@ -473,25 +480,7 @@ MicrobEalXshmRenderer::CreateSharedImageData(int width, int height, int depth, G
   mShmData.depth = depth;
   mShmData.data = (void*)data;
   mShmData.blocked = (int16_t*)blocked;
-
-  // Check Mozilla Web
-  NS_ENSURE_TRUE(mGWeb, NS_ERROR_FAILURE);
-  // Get Mozilla Web
-  GMozillaWeb* web = G_MOZILLA_WEB(mGWeb);
-
-  // Get Window List
-  GSList* window_list = (GSList*)web->window_list;
-
-  // Go Thru Window List and send updated shm buffer info
-  for (int i=0; g_mozilla_web_check_bus(web) && window_list; window_list = window_list->next, i++) {
-
-    GMozillaEngine* engine = G_MOZILLA_ENGINE(window_list->data);
-    if (!engine) continue;
-
-    engine->shmid = mShmData.id;
-    ShmInfoMessage msg = { (guint32) engine, mShmData.id, mShmData.width, mShmData.height, mShmData.depth };
-    g_web_bus_send_message(web->web_bus, G_MOZILLA_UI, SHM_INFO_MSG, &msg, sizeof(msg));
-  }
+  UpdateUIShmInfo();
 
   return NS_OK;
 }
@@ -594,16 +583,15 @@ MicrobEalXshmRenderer::draw_blocked_rect(GMozillaEngine *engine, int left, int t
   g_return_val_if_fail(engine != NULL, FALSE);
   g_return_val_if_fail(GTK_IS_MOZ_EMBED(engine->engine), FALSE);
 
-  // Check SHM ID
-  if (engine->shmid <= 0)
-    return FALSE;
-
   nsCOMPtr <nsIDOMWindow> window;
   MicrobEalXshmRenderer *mshm = (MicrobEalXshmRenderer*)g_object_get_data(G_OBJECT(engine->global), G_MOZILLA_PAINT_LISTENER);
 
   if (!mshm)
     return FALSE;
 
+ if (!mshm->mShmData.id)
+     return FALSE;
+
   if (!engine->is_active || mshm->mShmData.IsBlocked())
     return FALSE;
 
